//
//  AMWindow.m
//
//  Created by Andreas on 05.08.04.
//  Copyright 2004 Andreas Mayer. All rights reserved.
//
//	2004-08-15	Andreas Mayer
//	- calculate newFrame before sending the delegate windowShouldZoom:toFrame:


#import "AMWindow.h"

@interface NSWindow (ApplePrivate)
- (NSRect)_standardFrame;
- (void)_validSize:(NSSize *)frame force:(BOOL)force;
- (BOOL)_confirmSize:(NSSize *)frame force:(BOOL)force;
@end

@interface AMWindow (Private)
- (NSRect)userStateFrame;
- (void)setUserStateFrame:(NSRect)newUserStateFrame;
- (NSRect)windowWillUseStandardFrame:(NSWindow *)window defaultFrame:(NSRect)newFrame;
- (BOOL)windowShouldZoom:(NSWindow *)window toFrame:(NSRect)newFrame;
@end


@implementation AMWindow

- (id)initWithContentRect:(NSRect)contentRect styleMask:(NSUInteger)styleMask backing:(NSBackingStoreType)backingType defer:(BOOL)flag
{
	if (self = [super initWithContentRect:contentRect styleMask:(styleMask | NSResizableWindowMask) backing:backingType defer:flag]) {
		[self setCanResize:(styleMask & NSResizableWindowMask)];
	}
	return self;
}


// ============================================================
#pragma mark -
#pragma mark ━ accessor methods ━
// ============================================================

- (BOOL)canResize
{
	return _am_canResize;
}

- (void)setCanResize:(BOOL)newCanResize
{
	[self setShowsResizeIndicator:newCanResize];
	_am_canResize = newCanResize;
}

- (BOOL)canZoom
{
	return _am_canZoom;
}

- (void)setCanZoom:(BOOL)newCanZoom
{
	[[self standardWindowButton:NSWindowZoomButton] setEnabled:newCanZoom];
	_am_canZoom = newCanZoom;
}

- (NSRect)userStateFrame
{
	return _am_userStateFrame;
}

- (void)setUserStateFrame:(NSRect)newUserStateFrame
{
	_am_userStateFrame = newUserStateFrame;
}


// ============================================================
#pragma mark -
#pragma mark ━ window methods ━
// ============================================================

- (void)zoom:(id)sender
{
	if ([self canZoom]) {
		BOOL hasStandardFrame = NO;
		NSRect newFrame;
		// 1. get standard frame
		NSRect standardFrame;
		if ([self respondsToSelector:@selector(_standardFrame)]) {
			standardFrame = [self _standardFrame];
		} else {
			NSRect defaultFrame;
			defaultFrame = [[self screen] visibleFrame];
			defaultFrame.size.width -= 7;
			defaultFrame.size.height -= 7;
			[super _validSize:&defaultFrame.size force:NO];
			if ([[self delegate] respondsToSelector:@selector(windowWillUseStandardFrame:defaultFrame:)]) {
				standardFrame = [[self delegate] windowWillUseStandardFrame:self defaultFrame:defaultFrame];
			} else if ([self respondsToSelector:@selector(windowWillUseStandardFrame:defaultFrame:)]) {
				standardFrame = [self windowWillUseStandardFrame:self defaultFrame:defaultFrame];
			} else {
				standardFrame = defaultFrame;
			}
		}
		// 2. constrain default frame to screen
		standardFrame = [self constrainFrameRect:standardFrame toScreen:[self screen]];
		//NSLog(@"standardFrame x:%f y:%f w:%f h:%f", standardFrame.origin.x, standardFrame.origin.y, standardFrame.size.width, standardFrame.size.height);
		// 3. compare current frame with standard frame
		NSRect currentFrame = [self frame];
		//NSLog(@"currentFrame x:%f y:%f w:%f h:%f", currentFrame.origin.x, currentFrame.origin.y, currentFrame.size.width, currentFrame.size.height);
		hasStandardFrame = ((abs(currentFrame.size.width-standardFrame.size.width)<8) && (abs(currentFrame.size.height-standardFrame.size.height)<8));
		//NSLog(@"hasStandardFrame:%@", (hasStandardFrame ? @"YES" : @"NO"));

		// find new frame 
		if (hasStandardFrame) {
			newFrame = currentFrame;
			newFrame.size = [self userStateFrame].size;
			if ((newFrame.size.width == 0) || (newFrame.size.height == 0)) {
				// no user state
				newFrame = standardFrame;
			} else {
				newFrame.origin.y += currentFrame.size.height-newFrame.size.height;
			}
		} else {
			newFrame = standardFrame;
			// save user state
			//[self setUserStateFrame:currentFrame];
		}
		// move frame completely inside screen
		NSRect screenFrame = [[self screen] visibleFrame];
		newFrame.origin.x = MAX(NSMinX(screenFrame), NSMinX(newFrame));
		newFrame.origin.x = MIN(NSMinX(newFrame), NSMaxX(screenFrame)-newFrame.size.width);
		newFrame.origin.y = MAX(NSMinY(screenFrame), NSMinY(newFrame));
		newFrame.origin.y = MIN(NSMinY(newFrame), NSMaxY(screenFrame)-newFrame.size.height);
		
		// ask delegate
		BOOL shouldZoom = YES;
		if ([[self delegate] respondsToSelector:@selector(windowShouldZoom:toFrame:)]) {
			shouldZoom = [[self delegate] windowShouldZoom:self toFrame:newFrame];
		} else if ([self respondsToSelector:@selector(windowShouldZoom:toFrame:)]) {
			shouldZoom = [self windowShouldZoom:self toFrame:newFrame];
		}
		if (shouldZoom) {
			if (!hasStandardFrame) {
				// save user state
				[self setUserStateFrame:currentFrame];
			}
			_am_isZooming = YES;
			[self setFrame:newFrame display:YES animate:YES];
			_am_isZooming = NO;

			/*
			if (hasStandardFrame) {
				newFrame = currentFrame;
				newFrame.size = [self userStateFrame].size;
				if ((newFrame.size.width == 0) || (newFrame.size.height == 0)) {
					// no user state
					newFrame = standardFrame;
				} else {
					newFrame.origin.y += currentFrame.size.height-newFrame.size.height;
				}
			} else {
				newFrame = standardFrame;
				// save user state
				[self setUserStateFrame:currentFrame];
			}
			// move frame completely inside screen
			NSRect screenFrame = [[self screen] visibleFrame];
			newFrame.origin.x = MAX(NSMinX(screenFrame), NSMinX(newFrame));
			newFrame.origin.x = MIN(NSMinX(newFrame), NSMaxX(screenFrame)-newFrame.size.width);
			newFrame.origin.y = MAX(NSMinY(screenFrame), NSMinY(newFrame));
			newFrame.origin.y = MIN(NSMinY(newFrame), NSMaxY(screenFrame)-newFrame.size.height);
			//NSLog(@"newFrame x:%f y:%f w:%f h:%f", newFrame.origin.x, newFrame.origin.y, newFrame.size.width, newFrame.size.height);
			_am_isZooming = YES;
			[self setFrame:newFrame display:YES animate:YES];
			_am_isZooming = NO;
			 */
		}
	}
}

- (NSTimeInterval)animationResizeTime:(NSRect)newFrame
{
	NSTimeInterval result;
	if (_am_isZooming) {
		result = 0.1;
	} else {
		result = [super animationResizeTime:newFrame];
	}
	return result;
}

- (NSSize)windowWillResize:(NSWindow *)sender toSize:(NSSize)proposedFrameSize
{
	NSSize result = [self frame].size;
	if ([self canResize]) {
		result = proposedFrameSize;
	}
	return result;
}


@end
